/* * * * * * * * * * * * * * *INCLUDES * * * * * * * * * * * * * * * * * * */
#include <stdio.h>
#include <stdlib.h>
#include <sys/name.h>
#include <windows/Qwindows.h>
#include <string.h>
#include <sys/dev.h>
#include <sys/fd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/psinfo.h>
#include <sys/stat.h>
#include <errno.h>

#include "keyinject.h"

/* * * * * * * * * * * * * * *GLOBALS* * * * * * * * * * * * * * * * * * * */
int   task_wid;                       /* window id of task needing inputs */
unsigned short row = 0;               /* row to display dialog box     */
unsigned short col = 0;               /* column to display dialog box  */
int   command;                        /* command sent by creating task */

int   fd;                             /* fd that inserts into 'qnx/screen' input queue */

void position_key(char *new_row, char *new_col, char *new_wid, char *taskid);

void main(int argc, char **argv)
{
   int  go = 0;                           /* controls main loop - 0 = loop, otherwise terminate */
   int  pid;                              /* picture id of the Popup keyboard */
   int  wid;                              /* window id of the Popup keyboard */
   int  tid;                              /* id of the task that sent the message to GetEvent */
   int  sid;                              /* id of the window server */
   QW_EVENT_MSG msg;                      /* struct returned from GetEvent() */
   pid_t  screen_tid;                     /* id for the screen task */
   char  screen_code[5];                  /* string buffer to insert */
   pid_t  status;                         /* return codes from library functions */
   char *temp_row;
   char *temp_col; 
   char *temp_wid; 
   char *temp_tid;
   QW_RECT_AREA new_position;

   /* clear out message sent to key handler */
   memset(screen_code, 0, sizeof(screen_code));   

   /* should use argc to verify that enough parameters were passed in !! */

   /* update the global row, col, wid and tid */  	
   position_key(argv[1], argv[2], argv[3], argv[4]);

   /* attach a name to the window server */
   SetName("KeyBoard", "Hardware Simulation");
	
   /* connect to QNX window server */
   /* must be displayed.            */
   if(0 == (sid = GraphicsOpen(0)))
   {
      printf("\nError attaching to the Window Server" );
      abort();
   }

   screen_tid = qnx_name_locate(0,"qnx/screen", 10, NULL); 
   /* verify that the screen_key task is available */
   if (-1 == screen_tid)
   {
      printf("\nCan't locate the screen_key task!\n");
      abort();
   }

   pid = PictureOpen("GraphKeyb",  "PopKeyA.pict", NULL, 0, 0, NULL, "LRW");

   if (0 == pid)
   {
      /* can't open the picture so close it down */
      printf("\nCould not open the keyboard picture.\n");
      abort();
   }

   WindowAt((int)row, (int)col, "sS", NULL);
   wid = WindowOpen("GraphKey", 0, 0, "@PopKeyA.wnd&f", NULL, NULL, pid);

   if (0== wid)
   {
      /* can't open the window so close it down */
      printf("\nCould not open the keyboard window.\n");
      abort();
   }

   status = qnx_name_attach(0, KEYBOARD);
   if (-1 == status)
   {
      /* can't attach to name server so close  it down - no task */
      /* will be able to find me */
      WindowClose();
      PictureClose(pid);
      GraphicsClose(sid);
      printf("\nCould not attach Keyboard to name server\n");
      abort();
   }

   /* get a fd to the screen's input queue */
   fd = get_input_queue(screen_tid);
   
   while(0 == go)
   {
      tid = GetEvent(0, (char *) &msg,sizeof(msg));

      switch(Event(&msg))
      {
         case QW_CLICK:
            /* need to verify that the button pressed is legal */
            /* NOT DOING THE CHECK YET */

            screen_code[1] = 0;
            /* check if Close button selected */
            if (!strcmp("Close", msg.hdr.key))
            {
               /* close the window */
               go = 4;
               /* cause an enter key to be simulated */
               screen_code[0] = 0x0D;
            }
            else
            {
               /* translate tag to screen code */
               screen_code[0] = atoh(msg.hdr.key);
            }

            /* make the tasks window the current window (and active */

            WindowCurrent(task_wid);
            WindowState("A");

            insertit(screen_code);

            /* make the GraphKey the current window */
            WindowCurrent(wid); 
            WindowState("A"); 
            break;

         case QW_QUIT:
            /* event from Window Server to close it down */
            go = 1;
            break;
         case GK_CLOSE:
            /* message received from a task to close it down */
            go = 2;
            break;
         case GK_OPEN:
            /* get pointers to the row, col and wid passed via PutEvent() */
            temp_row = strtok(msg.hdr.key, " ");
            temp_col = strtok(NULL, " ");
            temp_wid = strtok(NULL, " ");
            temp_tid = strtok(NULL, " ");

            /* need to update new_position before changing it to new location */
            WindowInfo(NULL, &new_position, NULL, NULL, NULL, NULL);
            row = new_position.row;
            col = new_position.col;

            /* update the globals row, col, wid and tid*/
            position_key(temp_row, temp_col, temp_wid, temp_tid);

            if ( row != new_position.row || col != new_position.col ) {
               new_position.row = row;
               new_position.col = col;
               /* move the window */
               WindowChange(&new_position, 0, 0, QW_KEEP, QW_KEEP);
               }
            break;
         case QW_EDITED:
            break;
         default:
            printf("Not a window event\n");
            break;
      }
   }

   PictureClose(pid);
   WindowClose();
   GraphicsClose(sid);
}

/********************************************************************/
/* this routine changes the position of the keyboard due to */
/* a PutEvent from a task or upon initialization                    */
/********************************************************************/
void position_key(char *new_row, char *new_col, char *new_wid, char *taskid)
{
   int   row_value;
   int   col_value;
   int   wid_value;

   row_value = atoi(new_row);
   col_value = atoi(new_col);
   wid_value = atoi(new_wid);

   if (GK_KEEP != row_value)
   {
      row = row_value;
   }
   else if (GK_KEEP == row)
   {
      row = 700; /* default */
   }

   if (GK_KEEP != col_value)
   {
      col = col_value;
   }
   else if (GK_KEEP == col)
   {
      col = 700; /* default */
   }

   if (wid_value != GK_KEEP)
   {
      task_wid = wid_value;
   }
   else if (GK_KEEP == task_wid)
   {
      printf("Error - Graphical Keyboard needs the Window ID of receiving task\n");
   }
}

int get_input_queue(pid_t screen_tid)
{
struct _fd_entry buf;
struct _dev_info_entry info;
int keepgo = 1;
int fd;
int fdi = -1;

for ( fd=0; keepgo && ((fd = qnx_fd_query( 0, screen_tid, fd, &buf )) != -1); ++fd ) {
    if ( dev_fdinfo( buf.pid, screen_tid, fd, &info ) == 0  ) {
       if ( strcmp(info.driver_type, "console") == 0 && (buf.flags & _FD_CLOSE_ON_EXEC) )
          keepgo = 0;
       }
    }

/* now we've got one */
if ( keepgo == 0 ) {
   fdi = open(&info.tty_name[0], O_RDWR);
   }
if ( fdi == -1 ) {
   printf("unable to open input queue\n");
   abort();
   }
return(fdi);
}

insertit(char *str)
{
int ret;

ret = dev_insert_chars(fd,strlen(str),str);
if ( ret == -1 )
   printf("Error on insert: ret=%d, err=%s\n",ret, strerror(errno));
}

